#ifndef _ANIMATION_H_
#define _ANIMATION_H_

#include <cmath>
#ifndef M_PI
	#define M_PI 3.14159265358979323846f
#endif

#include <stdlib.h>
#include <time.h>
#include <iostream>
using namespace std;


#include "vec.h"
#include "dynamics.h"
#include "objLoader.h"
#include "skinning.h"


class Animation {
public :
	Mesh *clothMesh; // cloth model
	Mesh *characterMesh; // character model

	// display mass-spring system :
	bool drawMass;
	// display collision model :
	bool drawCollisionModel;
	// display visual model :
	bool drawVisualModel;

	Dynamics dynamics; // contains all the dynamics animation
	Skinning skinning; // contains all the skinning animation

	// Dynamics parameters :
	float oldTime;
	Vec initPos;
	unsigned int nbBalls;
	float mass;
	float radius;
	float stiffness;
	float initLength;
	float damping;

	// Skinning parameters :
	int indexCharacter; // index of the start of the character collision model in dynamics.positions
	std::vector<Vec> initPosCollisionModel; // initPosCollisionModel[i] = initial position (position at rest pose)
											// of sphere i of the character collision model
	std::vector<unsigned int> influences;	// influences[i] = index in the vectors of skinning 
											// of the bone influencing sphere i of the character collision model

	float dt;	// animation time step
	
	Animation() {

		drawMass = true;
		drawCollisionModel = true;
		drawVisualModel = true;

		oldTime=  0.0f;
		Dynamics dynamics;
		initPos = Vec(0.0, 5.0, 0.0);
		nbBalls = 9;
		mass = 1.0f;
		radius = 0.1f;
		stiffness = 30.0f;
		initLength = 0.2f;
		damping = 0.2f;

		indexCharacter=0;
		
		dt=0;
	}
	
	~Animation() {
	}

	/////////////////////////////////////////
	//  Animation methods
	////////////////////////////////////////
	void setDt(); // sets the time step
	void animateDynamics(); // compute one time step of dynamics
	void animateSkinning(); // compute one time step of skinning
	void updateDynamicsParameters(); // update parameters of dynamics
	void updateCollisionModelFromSkinning(glm::mat4 *m_transfo); // update collision model
									// in dynamics from the skinning animation


	////////////////////////////////////////
	// Initialization methods
	////////////////////////////////////////
	void initScene(unsigned int nfpp); // initialize the animation
	void setDynamics(); // initialize the dynamics
	void setSkinning(); // initialize the skinning
	void setInfluences(); // set the influences of skinning bones 
				// on collision model of dynamics


	///////////////////////////////////////////////
	//  Draw methods
	//////////////////////////////////////////////
	void drawGround();
	void drawSphere();
	void drawBalls(bool drawMass);
	void drawSprings();
	void renderScene(void) ;
};

#endif