#ifndef _SKINNING_H
#define _SKINNING_H

#include <vector>
#include "glm/glm.hpp"
#include "glm/gtc/type_ptr.hpp"
#include "glm/gtc/matrix_transform.hpp"

typedef glm::vec4 Point;
typedef glm::vec3 Vec3;


class Skinning {
public :
	unsigned int num_bones;					// number of bone in skinning

	std::vector<Vec3> jointsInitPos;		// initial position of bones
	std::vector<Vec3> jointsInitAxis;
	std::vector<float> jointsInitAngle;		// initial rotation (axis, angle) of bones

	std::vector<Vec3> jointsPos;			// current position of bones
	std::vector<Vec3> jointsAxis;
	std::vector<float> jointsAngle;			// current rotation (axis, angle) of bones

	std::vector<int> parent;				// parent[i] gives index in previous vectors of the parent of bone i

	
	unsigned int num_influences; // number of bones influencing each vertex (same for all vertices)
	unsigned int num_vertices; // number of vertices being skinned

	// Each pair (bone_index[num_influences*i+k], bone_weight[num_influences*i+k]) 
	// gives the information about the k-th influence on vertex i
	// (k in 0..num_influences-1)
	// bone_index[num_influences*i+k] gives the index of the bone influencing
	// bone_weight[num_influences*i+k] gives the weight of the influence
	std::vector<float> bone_index;	
	std::vector<float> bone_weight;


	Vec3 globalTra; // translation of center of gravity from rest pose
	std::vector<Vec3> dofs;	// dofs[i] = value of degrees of freedom of bone i = (rx, ry, rz)

public :
	Skinning() {
	}

	~Skinning() {
		jointsInitPos.clear();
		parent.clear();
		jointsInitAxis.clear();
		jointsInitAngle.clear();
	}

	void initModelData(unsigned int nv); // initialize the bones positions, orientations, influences and weights

	void animate(float dt); // compute one time step of skinning (modify globalTra and dofs)
	void updateJoints(); // update the current joint positions and orientations with respect to globalTra and dofs
};

#endif