#ifndef _VEC_H_
#define _VEC_H_

#include <math.h>
#include "glm/glm.hpp"
#include "glm/gtc/type_ptr.hpp"
#include "glm/gtc/matrix_transform.hpp"

typedef glm::vec4 Point;
typedef glm::vec3 Normal;

class Vec 
{
public:
	float x;
	float y;
	float z;

public:
	Vec() : x(0.0), y(0.0), z(0.0) {}
	Vec(float x0, float y0, float z0) : x(x0), y(y0), z(z0) {}
	Vec(const Vec& v) : x(v.x), y(v.y), z(v.z) {}
	Vec(Point p) : x(p.x), y(p.y), z(p.z) {}
	Vec(Normal p) : x(p.x), y(p.y), z(p.z) {}
	
	float operator* (const Vec& v) {
		return x*v.x + y*v.y + z*v.z;
	}

	friend Vec operator* (float f, const Vec& v);
	
	Vec operator* (float f) {
		return Vec(f*x, f*y, f*z);
	}

	Vec operator- (const Vec& v) {
		return Vec(x-v.x, y-v.y, z-v.z);
	}
	
	Vec operator+ (const Vec& v) {
		return Vec(x+v.x, y+v.y, z+v.z);
	}

	float norm() const { 
		return sqrt(x*x + y*y + z*z); 
	}

	float normalize() {
		const float n = norm();
		x /= n;
		y /= n;
		z /= n;
		return n;
    }

};


#endif // _VEC_H_
